from ..RequestHandler import RequestHandler
from datetime import datetime

class MatchApi:
    ENDPOINTS = {
        'BY_MATCH_ID': '/lol/match/v5/matches/{}',
        'BY_MATCH_ID_TIMELINE': '/lol/match/v5/matches/{}/timeline',
        'BY_PUUID_MATCHLIST': '/lol/match/v5/matches/by-puuid/{}/ids'
    }

    def __init__(self, region, api_key):
        self.request_handler = RequestHandler(api_key, region, True)

    def by_match_id(self, match_id): # Example of match_id : EUW1_6333379226
        return self.request_handler.make_request(self.ENDPOINTS['BY_MATCH_ID'].format(match_id))

    def by_match_id_timeline(self, match_id):
        return self.request_handler.make_request(self.ENDPOINTS['BY_MATCH_ID_TIMELINE'].format(match_id)) 

    def by_puuid_matchlist(
        self, 
        puuid: str,
        startTime: str = None,
        endTime: str = None,   
        queue: int = None,          # https://static.developer.riotgames.com/docs/lol/queues.json
        typeGame: str = None,       # ranked, normal, tourney, tutorial
        start: int = 0,
        count: int = 20,
    ):
        query_params = {k: v for k, v in locals().items() if v is not None and k != 'self' and k != 'puuid'}

        if startTime:
            temp_start = (datetime.now() - datetime.fromisoformat(startTime))
            query_params['startTime'] = int(temp_start.total_seconds())
        if endTime:
            temp_end = (datetime.now() - datetime.fromisoformat(endTime))
            query_params['endTime'] = int(temp_end.total_seconds())

        return self.request_handler.make_request(self.ENDPOINTS['BY_PUUID_MATCHLIST'].format(puuid), query_params=query_params)


from ..RequestHandler import RequestHandler

class LolStatusApi:
    ENDPOINTS = {
        'BY_REGION': '/lol/status/v4/platform-data'
    }

    def __init__(self, region, api_key):
        self.request_handler = RequestHandler(api_key, region, False)

    def region(self):
        return self.request_handler.make_request(self.ENDPOINTS['BY_REGION'])
    


from ..RequestHandler import RequestHandler

class LeagueApi:
    ENDPOINTS = {
        'BY_CHALL_QUEUE': '/lol/league/v4/challengerleagues/by-queue/{}',
        'BY_SUMMONER': '/lol/league/v4/entries/by-summoner/{}',
        'BY_QUEUE_TIER_DIVISION': '/lol/league/v4/entries/{}/{}/{}',
        'BY_GM_QUEUE': '/lol/league/v4/grandmasterleagues/by-queue/{}',
        'BY_LEAGUE': '/lol/league/v4/leagues/{}',
        'BY_MASTER_QUEUE': '/lol/league/v4/masterleagues/by-queue/{}'
    }

    def __init__(self, region, api_key):
        self.request_handler = RequestHandler(api_key, region, False)

    def chall_queue(self, queue): # queue = RANKED_SOLO_5x5, RANKED_FLEX_SR, RANKED_FLEX_TT
        return self.request_handler.make_request(self.ENDPOINTS['BY_CHALL_QUEUE'].format(queue))
    
    def summoner(self, summoner_id):
        return self.request_handler.make_request(self.ENDPOINTS['BY_SUMMONER'].format(summoner_id))
    
    def league_entries(self, queue,      # queue = RANKED_SOLO_5x5, RANKED_FLEX_SR, RANKED_FLEX_TT
                            tier,        # tier = DIAMOND, PLATINUM, GOLD, SILVER, BRONZE, IRON
                            division,    # division = I, II, III, IV
                            page=1): 
        
        query_params = {'page': page}
        return self.request_handler.make_request(self.ENDPOINTS['BY_QUEUE_TIER_DIVISION'].format(queue, tier, division), query_params=query_params)
    
    def gm_queue(self, queue):
        return self.request_handler.make_request(self.ENDPOINTS['BY_GM_QUEUE'].format(queue))
    
    def league_id(self, league_uuid):
        return self.request_handler.make_request(self.ENDPOINTS['BY_LEAGUE'].format(league_uuid))
    
    def master_queue(self, queue):
        return self.request_handler.make_request(self.ENDPOINTS['BY_MASTER_QUEUE'].format(queue))


from ..RequestHandler import RequestHandler

class SummonerApi:
    ENDPOINTS = {
        'BY_ACCOUNT': '/lol/summoner/v4/summoners/by-account/{}',
        'BY_NAME': '/lol/summoner/v4/summoners/by-name/{}',
        'BY_PUUID': '/lol/summoner/v4/summoners/by-puuid/{}',
        'BY_ID': '/lol/summoner/v4/summoners/{}'
    }
    
    def __init__(self, region, api_key):
        self.request_handler = RequestHandler(api_key, region, False)

    def by_account(self, encrypted_account_id):
        return self.request_handler.make_request(self.ENDPOINTS['BY_ACCOUNT'].format(encrypted_account_id))

    def by_name(self, summoner_name):
        return self.request_handler.make_request(self.ENDPOINTS['BY_NAME'].format(summoner_name))
    
    def by_puuid(self, encrypted_puuid):
        return self.request_handler.make_request(self.ENDPOINTS['BY_PUUID'].format(encrypted_puuid))

    def by_id(self, encrypted_summoner_id):
        return self.request_handler.make_request(self.ENDPOINTS['BY_ID'].format(encrypted_summoner_id))


import requests, requests_cache, time
from requests.exceptions import HTTPError
import logging
from urllib.parse import urlencode

logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

MAX_RETRIES = 3
CACHE_NAME = 'riot_api_cache'
EXPIRE_AFTER = 3600
ERROR_TIME_SLEEP = 3
logging.basicConfig(level=logging.INFO)

REGION_TO_PLATFORM = {
    'eun1': 'europe',
    'euw1': 'europe',
    'tr1': 'europe',
    'ru': 'europe',
    'jp1': 'asia',
    'kr': 'asia',
    'br1': 'americas',
    'la1': 'americas',
    'la2': 'americas',
    'na1': 'americas',
    'oc1': 'americas'
}


class RequestHandler:
    def __init__(self, api_key, region, use_platform, expire_after=3600, max_retries=3):
        self.api_key = api_key
        self.region = region
        self.use_platform = use_platform
        self.expire_after = expire_after
        self.max_retries = max_retries
        self.session = requests.Session()
        self.set_cache(expire_after)

        self.retries = 0
    
    def set_cache(self, expire_after, cache_name='riot_api_cache'):
        requests_cache.install_cache(cache_name, expire_after=expire_after)

    def build(self, region, endpoint, query_params=None):
        domain = REGION_TO_PLATFORM[region] if self.use_platform else region
        base_url = f"https://{domain}.api.riotgames.com{endpoint}"
        
        return base_url if not query_params else f'{base_url}?{urlencode(query_params)}'

    def make_request(self, endpoint, query_params=None):
        self.retries = 0
        url = self.build(self.region, endpoint, query_params)
        headers = {'X-Riot-Token': self.api_key}

        while self.retries < self.max_retries:
            try:
                response = self.session.get(url, headers=headers)
                response.raise_for_status()

                return response.json()
            except requests.exceptions.RequestException as e:
                RequestHandler.handle_error(e)
                self.retries += 1
                logging.info(f"Retrying request {self.retries}/{self.max_retries}")

    def handle_error(exception):
        if isinstance(exception, requests.exceptions.ConnectionError):
            logging.info("Unable to connect to the server:", str(exception))
        elif isinstance(exception, requests.exceptions.HTTPError):
            logging.info("HTTP error occurred:", str(exception))

            if exception.response.status_code == 429:
                retry_after = exception.response.headers['Retry-After']
                logging.info(f"Waiting {retry_after} seconds before retrying...")
                
                time.sleep(int(retry_after))
        else:
            logging.info("An unexpected error occurred:", str(exception))



from setuptools import setup, find_packages

# Read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='PoroPilot',
    version='0.2.8',
    description='Wrapper Interface for Riot API',
    url='https://github.com/anuiit/PoroPilot',
    author='anuiit',
    author_email='',
    keywords=['Riot', 'API', 'wrapper', 'Poro', 'Pilot', 'League of Legends', 'LoL', 'game', 'gaming', 'esports', 'riotgames', 'riot-api', 'riot-api-wrapper', 'riot-api-python', 'riot-api'],
    license='MIT',
    packages=find_packages(),
    install_requires=['requests', 'requests-cache'],
    long_description=long_description,
    long_description_content_type='text/markdown',
)


from ..RequestHandler import RequestHandler

class ChampionApi:
    ENDPOINTS = {
        'CHAMP_ROTATION': '/lol/platform/v3/champion-rotations'
    }

    def __init__(self, region, api_key):
        self.request_handler = RequestHandler(api_key, region, False)

    def champion_rotation(self):
        return self.request_handler.make_request(self.ENDPOINTS['CHAMP_ROTATION'])


from ..RequestHandler import RequestHandler

class ChampionMasteryApi:
    ENDPOINTS = {
        'BY_PUUID': '/lol/champion-mastery/v4/champion-masteries/by-puuid/{}',
        'BY_PUUID_CHAMPION': '/lol/champion-mastery/v4/champion-masteries/by-puuid/{}/by-champion/{}',
        'BY_PUUID_TOP_CHAMPIONS': '/lol/champion-mastery/v4/champion-masteries/by-puuid/{}/top',
        'BY_PUUID_SUM_SCORE': '/lol/champion-mastery/v4/scores/by-puuid/{}',
    }

    def __init__(self, region, api_key):
        self.request_handler = RequestHandler(api_key, region, False)

    def by_puuid(self, puuid):
        return self.request_handler.make_request(self.ENDPOINTS['BY_PUUID'].format(puuid))

    def by_puuid_champion(self, puuid, champion_id):
        return self.request_handler.make_request(self.ENDPOINTS['BY_PUUID_CHAMPION'].format(puuid, champion_id))

    def by_puuid_top_champions(self, summoner_id, count=3):
        query_params = {k: v for k, v in locals().items() if v is not None and k != 'self'}
        return self.request_handler.make_request(self.ENDPOINTS['BY_PUUID_TOP_CHAMPIONS'].format(summoner_id))
    
    def by_puuid_sum_score(self, summoner_id, champion_id):
        return self.request_handler.make_request(self.ENDPOINTS['BY_PUUID_SUM_SCORE'].format(summoner_id, champion_id))
    


from .Endpoints import *

class PoroPilot:
    def __init__(self, api_key, region):
        # TODO: add api class
        # self.api = Api(region, api_key)
        # self.summoner = SummonerApi()

        self.summoner = summoner_v4.SummonerApi(region, api_key)
        self.match = match_v5.MatchApi(region, api_key)
        self.champion = champion_v3.ChampionApi(region, api_key)
        self.mastery = championMastery_v4.ChampionMasteryApi(region, api_key)
        self.league = league_v4.LeagueApi(region, api_key)
        self.spectator = spectator_v5.SpectatorApi(region, api_key)
        self.league_exp = leagueExp_v4.LeagueExpApi(region, api_key)
        self.lol_status = lolStatus_v4.LolStatusApi(region, api_key)
        self.account = account_v1.AccountApi(region, api_key)
        
        


    
    
    

    
    # get winrate ?
    # general functions to get stats ?
    # get games played with count ?
    # stats by champion ?
    # stats by role ?
    # average stats ?
    # random games picked ?
    # change champion id to champion name ?